//////////////////////////////////////////////////////////////////////
// A sample code to implement the model averaging estimators of
// Kuersteiner and Okui "Instrument Selection by First Stage
// Prediction Averaging" (2009)
// The class file that defines the classes of the estimators is
// "wiv_v1.ox". The current code provides an example of the use of
// those estimator classes.
// The code and the class file are provided free of charge and `as is'.
// It comes without any guarantee to be free of errors nor for its
// performance. The code may only be used for academic
// and/or instructional purposes.
// This version: 2009/APR/29
// Written by Ryo Okui (okui@ust.hk)	
//////////////////////////////////////////////////////////////////////

#include <oxstd.h>
#include "wiv_v1.ox" // Include the class file.

main()
{

// generating a data set
		decl vy, vx, mZ, mU; 
		decl mSigma = ones(2,2); mSigma[0][1] = 0.5; mSigma [1][0] =0.5;
		decl gamma = ones(20,1);
		decl mSigmac; mSigmac = choleski(mSigma); 
		mZ = rann(100,20); mU = rann(100,2)*(mSigmac');
		vx = mZ*gamma + mU[][1];
		vy = 0.1*vx  + mU[][0]; 

		decl myest;
		myest = new Tsls(vy,vx,mZ);	// construct a Tsls class.
									// Tsls(y,x,Z). y: dependent variable; x: regressors; Z: instrument. 
		myest.Estimate();			// Estimate the parameter by 2SLS.
		print("2SLS-All", myest.GetEstimate());	// Print the value of the 2SLS estimator.
		print("S.E.", myest.Stdb());		// Print the heteroskedastic robust standard error.
		delete myest;				// delete the class.
		myest = new Liml(vy,vx,mZ);	//construct a Liml class.
									// Liml(y,x,Z). y: dependent variable; x: regressors; Z: instrument. 
		myest.Estimate();			// Estimate the parameter by LIML
		print("Liml-All", myest.GetEstimate());	// Print the value of the LIML estimator.
		print("S.E.", myest.Stdb());		// Print the heteroskedastic robust standard error.
		delete myest;				// delete the class.
		myest = new Fuller(vy,vx,mZ,1);	//construct a Fuller class.
									// Fuller(y,x,Z,a). y: dependent variable; x: regressors; Z: instrument; a: the parameter in the Fuller estimator
		myest.Estimate();			// Estimate the parameter by Fuller.
		print("Fuller-All", myest.GetEstimate());	// Print the value of the Fuller estimator.
		print("S.E.", myest.Stdb());		// Print the heteroskedastic robust standard error.
		delete myest;

		myest = new MATsls(vy,vx,mZ);	// construct a MATsls class.
									// MATsls(y,x,Z). y: dependent variable; x: regressors; Z: instrument. 
		myest.SetW(1,1);			  // Set the weight.
									// SetW(l,type). l: lambda; type: 0 sequential instrument selection.
									// 1: 2SLS-P, 2: 2SLS-C, 3: 2SLS-U, 4: 2SLS-Ps.
		myest.Estimate();
		print("2SLS-P", myest.GetEstimate());	// Print the value of the MA2SLS estimator.
		print("S.E.", myest.Stdb());		// Print the heteroskedastic robust standard error.
		print("KW+", myest.GetKWp());			// Print KW+ (see the paper)
		print("KW-", myest.GetKWn());			// Print KW- (see the paper)
		print("Pseudo R2", myest.GetR2());	// Print the pseudo R2.	This function can be used only when the number of regressors is 1.
		delete myest;
		myest = new MALiml(vy,vx,mZ);	// construct a MALiml class.
									// MALiml(y,x,Z). y: dependent variable; x: regressors; Z: instrument. 
		myest.SetW(1,2);			  // Set the weight.
									// SetW(l,type). l: lambda; type: 0 sequential instrument selection.
									// 1: LIML-P, 2: LIML-C, 3: LIML-U.
		myest.Estimate();
		print("LIML-C", myest.GetEstimate());	// Print the value of the MALiml estimator.
		print("S.E.", myest.Stdb());		// Print the heteroskedastic robust standard error.
		print("KW+", myest.GetKWp());			// Print KW+ (see the paper)
		print("KW-", myest.GetKWn());			// Print KW- (see the paper)
		delete myest;
		myest = new MAFuller(vy,vx,mZ,1);	//construct a Fuller class.
									// MAFuller(y,x,Z,a). y: dependent variable; x: regressors; Z: instrument; a: the parameter in the Fuller estimator
		myest.SetW(1,3);			  // Set the weight.
									// SetW(l,type). l: lambda; type: 0 sequential instrument selection.
									// 1: Fuller-P, 2: Fuller-C, 3: Fuller-U.
		myest.Estimate();
		print("Fuller-U", myest.GetEstimate());	// Print the value of the MAFuller estimator.
		print("S.E.", myest.Stdb());		// Print the heteroskedastic robust standard error.
		print("KW+", myest.GetKWp());			// Print KW+ (see the paper)
		print("KW-", myest.GetKWn());			// Print KW- (see the paper)
		delete myest;

}