//////////////////////////////////////////////////////////////////////
// This code defines classes for the estimators of 
// Kuersteiner and Okui "Instrument Selection by First Stage
// Prediction Averaging" (2009)
// A sample Ox code for the use of these classes is
// "sample_code_for_wiv.ox".
// The code and the class file are provided free of charge and `as is'.
// It comes without any guarantee to be free of errors nor for its
// performance. The code may only be used for academic
// and/or instructional purposes.
// This version: 2009/APR/29
// Written by Ryo Okui (okui@ust.hk)	
//////////////////////////////////////////////////////////////////////


#include <oxstd.h>
#import <maximize>
#import <solveqp>

class Tsls
{
	decl vy, vx, mZ, n;
	decl betatsls, mvhat;

	Tsls(const y, const x, const Z);
	virtual Estimate();
	virtual	Stdb();
	GetEstimate();
}	

Tsls::Tsls(const y, const x, const Z)
{
	vy = y; vx = x; mZ = Z; n = rows(vx);   
}

Tsls::Estimate()
{
	betatsls =invertsym(vx'mZ*invertsym(mZ'mZ)*mZ'vx)*vx'mZ*invertsym(mZ'mZ)*mZ'vy; 
}

Tsls::Stdb()
{
  decl xhat;
  xhat = mZ*invertsym(mZ'mZ)*mZ'vx;
  decl ve = vy-vx*(betatsls);
  decl vhat = xhat.*ve;
  mvhat = invertsym(xhat'vx) *(vhat'vhat) *invertsym(vx'xhat);
  return sqrt(diagonal(mvhat))';
}

Tsls::GetEstimate()
{
	return betatsls;
}

///////////////////////////////////////////////////
class MATsls : Tsls
{

	decl weight, k, mK, mU, vl, type;
	decl sigmau, sigmae, sigmaue ; 
	decl mGmin, mGmax, mxpx, mxpy, mypy;
	MATsls(const y, const x, const Z);
	Firststage();
	Estimate();
	Stdb();
	GetR2();
	GetKWp();
	GetKWn();
	
	SetW(const sl, const st);
	
}

MATsls::GetKWp()
{
	return (maxc((weight~zeros(k,1))'))*mK;
}
MATsls::GetKWn()
{
	return fabs((minc((weight~zeros(k,1))'))*mK);
}

MATsls::GetR2()
{
	decl mpwx;
	if (columns(vx) ==1){
	mpwx = zeros(n,columns(vx));
	decl j;	for (j=0;j<k;j++){
	mpwx = mpwx + weight[j]*mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vx;
	}

	return (mpwx'vx)^2/(mpwx'mpwx*vx'vx);
	}
	else {
		return 0;
	}
}

MATsls::MATsls(const y, const x, const Z)
{
	Tsls(y,x,Z); k = columns(mZ); mK = range(1,k)';
    mGmin = mK*ones(1,k) .< ones(k,1)*(mK') .? mK*ones(1,k) .: ones(k,1)*(mK');
	mGmax = mK*ones(1,k) .> ones(k,1)*(mK') .? mK*ones(1,k) .: ones(k,1)*(mK');

}

MATsls::SetW(const sl, const st)
{
	vl = sl; type = st;
	decl vlx = 	vx*vl;
	mxpx = zeros(k,1); mxpy = zeros(k,1); mypy = zeros(k,1);
	decl j; for (j=0;j<k;j++)
	{
		mxpx[j] = vlx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vlx;
		mxpy[j] = vlx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy;
		mypy[j] = vy'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy;
	}
	mU = mK*ones(1,k) .> ones(k,1)*(mK') .? mxpx*ones(1,k) .: ones(k,1)*(mxpx');
	mU = vlx'vlx - mU;
	mU = mU - mU[k-1][k-1]*ones(k,k);

	Firststage();

	if (type==0){ //squential

	decl dnc; dnc=zeros(k,2); dnc[][0] = range(0,k-1)';
	decl j;
    for (j=0; j<k ; j++) {
       dnc[j][1] = (sigmaue^2)*((j+1)^2) + sigmae*(mU[j][j] + sigmau*(j+1));
       }
	dnc = sortbyc(dnc,1); 
	decl optk = dnc[0][0];
	weight = zeros(k,1); weight[optk]=1;

	}

	
	else if (type==1){ // 2SLS-P

	decl vlo = zeros(k,1);//-ones(k,1);
	decl vhi = ones(k,1);
    decl ir;
    decl mK = range(1,k)';
    decl mG, vG, mC, vD;
    mG = 2*((sigmaue^2)*mK*(mK') + (sigmaue^2)*mGmin + sigmae*mU  );
    vG = -mK*2*(sigmae*sigmau +4*sigmaue^2 ) + 2*sigmau*sigmae*mK ;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
   [ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,vlo,vhi);

	}
	
	else if (type==2){ // 2SLS-C

    decl vlo = -ones(k,1);
	decl vhi = ones(k,1);
    decl ir;
    decl mK = range(1,k)';
    decl mG, vG, mC, vD;
    mG = 2*((sigmaue^2)*mK*(mK') + (sigmaue^2)*mGmin + sigmae*mU);
    vG = -mK*2*(sigmae*sigmau + 4*sigmaue^2) + 2*sigmau*sigmae*mK ;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
   [ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,vlo,vhi);

	
	}
	
	else if (type==3){ // 2SLS-U

	decl ir;
    decl mK = range(1,k)';
    decl mG, vG, mC, vD;
    mG = 2*((sigmaue^2)*mK*(mK') + (sigmaue^2)*mGmin + sigmae*mU );
    vG = -mK*2*(sigmae*sigmau + 4*sigmaue^2) + 2*sigmau*sigmae*mK ;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
   [ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,<>,<>);

	}

	else if (type==4){ // 2SLS-Ps

    decl vlo = zeros(k,1); decl vhi = ones(k,1);
    decl ir;
    decl mK = range(1,k)';
    decl mG, vG, mC, vD;
    mG = 2*((sigmaue^2)*mK*(mK') + sigmae*mU - sigmae*sigmau*mGmin);
    vG = 2*sigmau*sigmae*mK;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
	[ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,vlo,vhi);

	}

	
	
}

MATsls::Firststage()
{
// Mallow's criterion for the first stage //
   decl vlx = vx*vl;
   sigmau = (vlx'vlx - vlx'mZ*invertsym(mZ'mZ)*mZ'vlx)/n;

     decl dnc; dnc=zeros(k,2); dnc[][0] = range(0,k-1)';
     decl j; for (j=0;j<k;j++){
     dnc[j][1]=mU[j][j] + 2*(j+1)*sigmau;
     }
     dnc = sortbyc(dnc,1); decl optk = dnc[0][0];
     decl vxhat; vxhat =  mZ[][:optk]*invertsym(mZ[][:optk]'mZ[][:optk])*(mZ[][:optk]'vlx);
     decl delta;  delta =invertsym(vx'mZ[][:optk]*invertsym(mZ[][:optk]'mZ[][:optk])*mZ[][:optk]'vx)*(vx'mZ[][:optk]*invertsym(mZ[][:optk]'mZ[][:optk])*mZ[][:optk]'vy);

// minimizing MSE //   
   sigmae = (vy - vx*delta)'(vy - vx*delta)/n; 
   sigmau = (vlx - vxhat)'(vlx - vxhat)/n;
   sigmaue=(vlx - vxhat)'(vy - vx*delta)/n; 

}



MATsls::Estimate()
{
	decl mxpwy, mxpwx;
	mxpwy = zeros(columns(vx),1); mxpwx = zeros(columns(vx),columns(vx));
    decl j; for (j=0;j<k;j++){
	mxpwx = mxpwx + weight[j]*vx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vx;
	mxpwy = mxpwy + weight[j]*vx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy;
     }
	 
	 
	betatsls = invertsym(mxpwx)*mxpwy;
}

MATsls::Stdb()
{
	decl mpwx;
	mpwx = zeros(n,columns(vx));
	decl j;	for (j=0;j<k;j++){
	mpwx = mpwx + weight[j]*mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vx;
	}
	
	decl ve = vy-vx*(betatsls);

	decl vhat; vhat = mpwx.*ve;
	decl mvhat = invertsym(mpwx'vx)*(vhat'vhat) *invertsym(vx'mpwx);
	return sqrt(diagonal(mvhat))';
}



////////////////////////////////////////////////////////////////////////////////

//////////// LIML ////////////////////////////
class Liml
{
	decl vy, vx, mZ; decl n;
	decl lambda, betaliml;

	Liml(const y, const x, const Z);
	virtual Estimate();
	fLiml(const vP, const adFunc, const avScore, const amHessian);
	virtual	Stdb();
	GetEstimate();
	GetLambda();
}	

Liml::Liml(const y, const x, const Z)
{
	vy = y; vx = x; mZ = Z;	n=rows(vx);

}

Liml::fLiml(const vP, const adFunc, const avScore, const amHessian)
{
	decl fA, fB, fAd, fBd, fAdd, fBdd;	
	fA = (vy - vx*vP)'mZ*invert(mZ'mZ)*mZ'(vy - vx*vP);
	fB = (vy - vx*vP)'(vy - vx*vP);
	adFunc[0]= - fA / fB;
	
return 1;
}

Liml::Estimate()
{
	decl ir, vp, dfunc;
	vp = 0.5*ones(columns(vx),1);	// starting value
	//MaxControl(1,-1);
	ir = MaxBFGS(fLiml, &vp, &dfunc, 0, TRUE);
	betaliml = vp; lambda = - dfunc;
}

Liml::GetEstimate()
{
	return betaliml;
}

Liml::GetLambda()
{
	return lambda;
}

Liml::Stdb()
{
  decl xhat;
  xhat = mZ*invertsym(mZ'mZ)*mZ'vx - lambda*vx;
  decl ve = vy-vx*betaliml;
  decl vhat = xhat.*ve;
  decl mvhat = invertsym(xhat'vx) *(vhat'vhat) *invertsym(vx'xhat);
  return sqrt(diagonal(mvhat))';
}

//////////// Model averagin LIML//////////////////
class MALiml : Liml
{

	decl weight, k, xhat, mK, mGamma, vl, type;
	decl sigmau, sigmae, sigmaue ; 
	decl mGmin, mGmax, mU;
	decl mxpx, mxpy, mypy;

	MALiml(const y, const x, const Z);
	Firststage();
	Estimate();
	Stdb();
 	GetKWp();
	GetKWn();

	SetW(const sl, const st);
	
}

MALiml::MALiml(const y, const x, const Z)
{
	Liml(y,x,Z); k = columns(mZ);
	mK = range(1,k)';
    mGmin = mK*ones(1,k) .< ones(k,1)*(mK') .? mK*ones(1,k) .: ones(k,1)*(mK');
	mGmax = mK*ones(1,k) .> ones(k,1)*(mK') .? mK*ones(1,k) .: ones(k,1)*(mK');
	
}

MALiml::SetW(const sl, const st)
{
	vl = sl; type = st;
	decl vlx = 	vx*vl;
	mxpx = zeros(k,1); mxpy = zeros(k,1); mypy = zeros(k,1);
	decl j; for (j=0;j<k;j++)
	{
		mxpx[j] = vlx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vlx;
		mxpy[j] = vlx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy;
		mypy[j] = vy'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy;
	}
	mU = mK*ones(1,k) .> ones(k,1)*(mK') .? mxpx*ones(1,k) .: ones(k,1)*(mxpx');
	mU = vlx'vlx - mU;
	mU = mU - mU[k-1][k-1]*ones(k,k);

	Firststage();

	if (type==0){ //squential

	decl dnc; dnc=zeros(k,2); dnc[][0] = range(0,k-1)';
	decl j;
    for (j=0; j<k ; j++) {
       dnc[j][1] = (2*sigmae*sigmau-sigmaue^2)*(j+1) + sigmae*(mU[j][j] - (mU[k-1][k-1]) );
       }
	dnc = sortbyc(dnc,1); 
	decl optk = dnc[0][0];
	weight = zeros(k,1); weight[optk]=1;

	}

	
	else if (type==1){ // LIML-P

    decl vlo = zeros(k,1); decl vhi = ones(k,1);
    decl ir;
    decl mG, vG, mC, vD;

    mG = 2*((-sigmaue^2)*mGmin + sigmae*mU - sigmae*mU[k-1][k-1]);
    vG = 2*sigmae*sigmau*mK;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
   [ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,vlo,vhi);

	}
	
	else if (type==2){ // LIML-C

    decl vlo = -ones(k,1); decl vhi = ones(k,1);
    decl ir;
    decl mG, vG, mC, vD;

    mG = 2*((-sigmaue^2)*mGmin + sigmae*mU - sigmae*mU[k-1][k-1]);
    vG = 2*sigmae*sigmau*mK;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
	[ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,vlo,vhi);
	
	}
	
	else if (type==3){ // LIML-U

	decl ir;
    decl mK = range(1,k)';
    decl mG, vG, mC, vD;
    mG = 2*((-sigmaue^2)*mGmin + sigmae*mU - sigmae*mU[k-1][k-1]);
    vG = 2*sigmae*sigmau*mK;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
   [ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,<>,<>);

	}
	
}


MALiml::GetKWp()
{
	return (maxc((weight~zeros(k,1))'))*mK;
}
MALiml::GetKWn()
{
	return fabs((minc((weight~zeros(k,1))'))*mK);
}


MALiml::Firststage()
{
// Mallow's criterion for the first stage //
  decl vlx = vx*vl;
   sigmau = (vlx'vlx - vlx'mZ*invertsym(mZ'mZ)*mZ'vlx)/n;

     decl dnc; dnc=zeros(k,2); dnc[][0] = range(0,k-1)';
     decl j; for (j=0;j<k;j++){
     dnc[j][1]=mU[j][j] + 2*(j+1)*sigmau;
     }
     dnc = sortbyc(dnc,1); decl optk = dnc[0][0];
     decl vxhat; vxhat =  mZ[][:optk]*invertsym(mZ[][:optk]'mZ[][:optk])*(mZ[][:optk]'vlx);
	decl delta;  
	decl firstliml; firstliml = new Liml(vy,vx,mZ[][:optk]);
	firstliml.Estimate();
	delta = firstliml.GetEstimate();
	delete firstliml;
	
	sigmae = (vy - vx*delta)'(vy - vx*delta)/n; 
	sigmau = (vlx - vxhat)'(vlx - vxhat)/n;
	sigmaue=(vlx - vxhat)'(vy - vx*delta)/n; 

}


MALiml::Estimate()
{
	decl mxpwy, mxpwx;
	decl betaj;
	mxpwy = zeros(columns(vx),1); mxpwx = zeros(columns(vx),columns(vx));
    decl j; for (j=0;j<k;j++){
	betaj = new Liml(vy,vx,mZ[][:j]);
	betaj.Estimate();
	mxpwx = mxpwx + weight[j]*(vx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vx - (betaj.GetLambda())*vx'vx);
	mxpwy = mxpwy + weight[j]*(vx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy- (betaj.GetLambda())*vx'vy);
	delete betaj;
	}
	 
	 
	betaliml = invertsym(mxpwx)*mxpwy;
}


MALiml::Stdb()
{
	decl mpwx;
	mpwx = zeros(n,columns(vx));
	decl betaj;
	decl j;	for (j=0;j<k;j++){
	betaj = new Liml(vy,vx,mZ[][:j]);
	betaj.Estimate();
	mpwx = mpwx + weight[j]*(mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vx - (betaj.GetLambda())*vx);
	delete betaj;
	}
	
	decl ve = vy-vx*betaliml;

	decl vhat; vhat = mpwx.*ve;
	decl mvhat = invertsym(mpwx'vx)*(vhat'vhat) *invertsym(vx'mpwx);
	return sqrt(diagonal(mvhat))';
}


////



//////////// Fuller ////////////////////////////
class Fuller
{
	decl vy, vx, mZ; decl n;
	decl lambda, betaFuller;
	decl alpha;
	Fuller(const y, const x, const Z, const a);
	virtual Estimate();
	virtual	Stdb();
	GetEstimate();
	GetLambda();
}	

Fuller::Fuller(const y, const x, const Z, const a)
{
	vy = y; vx = x; mZ = Z;	n=rows(vx);	alpha =a;

}

Fuller::Estimate()
{

	decl limlest; limlest = new Liml(vy,vx,mZ);
	limlest.Estimate();
	lambda = limlest.GetLambda();
	delete limlest;
	
	lambda = (lambda - (alpha/(n-columns(mZ)))*(1-lambda))/(1- (alpha/(n-columns(mZ)))*(1-lambda) );
	decl xhat = mZ*invert(mZ'mZ)*mZ'vx - lambda*vx;
	betaFuller = invertsym(xhat'vx)*(xhat'vy); 
}

Fuller::GetEstimate()
{
	return betaFuller;
}

Fuller::GetLambda()
{
	return lambda;
}

Fuller::Stdb()
{
  decl xhat;
  xhat = mZ*invertsym(mZ'mZ)*mZ'vx - lambda*vx;
  decl ve = vy-vx*betaFuller;
  decl vhat = xhat.*ve;
  decl mvhat = invertsym(xhat'vx) *(vhat'vhat) *invertsym(vx'xhat);
  return sqrt(diagonal(mvhat))';
}

//////////// Model averaging Fuller//////////////////
class MAFuller : Fuller
{

	decl weight, k, mE, xhat, mK, mGamma, vl, type;
	decl sigmau, sigmae, sigmaue ; 
	decl mGmin, mGmax, mU;
	decl mxpx, mxpy, mypy, vlam, vbeta;

	MAFuller(const y, const x, const Z, const a);
	Firststage();
	Estimate();
	Stdb();
 	GetKWp();
	GetKWn();

	SetW(const sl, const st);
	
	SelWDN();
	SelWMA();
	SelWMAC();
	SelWMAU();
	SelUBo(const se, const su, const sue, const vpi);
}

MAFuller::MAFuller(const y, const x, const Z, const a)
{
	Fuller(y,x,Z,a); k = columns(mZ); decl Fullerest;
	mK = range(1,k)';
    mGmin = mK*ones(1,k) .< ones(k,1)*(mK') .? mK*ones(1,k) .: ones(k,1)*(mK');
	mGmax = mK*ones(1,k) .> ones(k,1)*(mK') .? mK*ones(1,k) .: ones(k,1)*(mK');
}

MAFuller::SetW(const sl, const st)
{
	vl = sl; type = st;
	decl vlx = 	vx*vl;
	mxpx = zeros(k,1); mxpy = zeros(k,1); mypy = zeros(k,1);
	decl j; for (j=0;j<k;j++)
	{
		mxpx[j] = vlx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vlx;
		mxpy[j] = vlx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy;
		mypy[j] = vy'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy;
	}
	mU = mK*ones(1,k) .> ones(k,1)*(mK') .? mxpx*ones(1,k) .: ones(k,1)*(mxpx');
	mU = vlx'vlx - mU;
	mU = mU - mU[k-1][k-1]*ones(k,k);

	Firststage();

	if (type==0){ //squential

	decl dnc; dnc=zeros(k,2); dnc[][0] = range(0,k-1)';
	decl j;
    for (j=0; j<k ; j++) {
       dnc[j][1] = (2*sigmae*sigmau-sigmaue^2)*(j+1) + sigmae*(mU[j][j] - (mU[k-1][k-1]) );
       }
	dnc = sortbyc(dnc,1); 
	decl optk = dnc[0][0];
	weight = zeros(k,1); weight[optk]=1;

	}

	
	else if (type==1){ // LIML-P

    decl vlo = zeros(k,1); decl vhi = ones(k,1);
    decl ir;
    decl mG, vG, mC, vD;

    mG = 2*((-sigmaue^2)*mGmin + sigmae*mU - sigmae*mU[k-1][k-1]);
    vG = 2*sigmae*sigmau*mK;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
   [ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,vlo,vhi);

	}
	
	else if (type==2){ // LIML-C

    decl vlo = -ones(k,1); decl vhi = ones(k,1);
    decl ir;
    decl mG, vG, mC, vD;

    mG = 2*((-sigmaue^2)*mGmin + sigmae*mU - sigmae*mU[k-1][k-1]);
    vG = 2*sigmae*sigmau*mK;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
	[ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,vlo,vhi);
	
	}
	
	else if (type==3){ // LIML-U

	decl ir;
    decl mK = range(1,k)';
    decl mG, vG, mC, vD;
    mG = 2*((-sigmaue^2)*mGmin + sigmae*mU - sigmae*mU[k-1][k-1]);
    vG = 2*sigmae*sigmau*mK;
    mC = ones(1,k);
    vD =1; 
    weight = ones(k,1)/k;	// starting value
 
   [ir,weight]= SolveQP(mG,vG,<>,<>,mC,vD,<>,<>);

	}
	
}

MAFuller::GetKWp()
{
	return (maxc((weight~zeros(k,1))'))*mK;
}
MAFuller::GetKWn()
{
	return fabs((minc((weight~zeros(k,1))'))*mK);
}

MAFuller::Firststage()
{
// Mallow's criterion for the first stage //
  decl vlx = vx*vl;
   sigmau = (vlx'vlx - vlx'mZ*invertsym(mZ'mZ)*mZ'vlx)/n;

     decl dnc; dnc=zeros(k,2); dnc[][0] = range(0,k-1)';
     decl j; for (j=0;j<k;j++){
     dnc[j][1]=mU[j][j] + 2*(j+1)*sigmau;
     }
     dnc = sortbyc(dnc,1); decl optk = dnc[0][0];
     decl vxhat; vxhat =  mZ[][:optk]*invertsym(mZ[][:optk]'mZ[][:optk])*(mZ[][:optk]'vlx);
	decl delta;  
	decl firstfuller; firstfuller = new Fuller(vy,vx,mZ[][:optk],alpha);
	firstfuller.Estimate();
	delta = firstfuller.GetEstimate();
	delete firstfuller;
	
	sigmae = (vy - vx*delta)'(vy - vx*delta)/n; 
	sigmau = (vlx - vxhat)'(vlx - vxhat)/n;
	sigmaue=(vlx - vxhat)'(vy - vx*delta)/n; 

}






MAFuller::Estimate()
{
	decl mxpwy, mxpwx;
	decl betaj, betal;
	mxpwy = zeros(columns(vx),1); mxpwx = zeros(columns(vx),columns(vx));
    decl j; for (j=0;j<k;j++){
	betaj = new Fuller(vy,vx,mZ[][:j],alpha);
	betaj.Estimate();
	mxpwx = mxpwx + weight[j]*(vx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vx - (betaj.GetLambda())*vx'vx);
	mxpwy = mxpwy + weight[j]*(vx'mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vy- (betaj.GetLambda())*vx'vy);
	delete betaj;
	}
	 
	 
	betaFuller = invertsym(mxpwx)*mxpwy;
}


MAFuller::Stdb()
{
	decl mpwx;
	mpwx = zeros(n,columns(vx));
	decl betaj;
	decl j;	for (j=0;j<k;j++){
	betaj = new Fuller(vy,vx,mZ[][:j],alpha);
	betaj.Estimate();
	mpwx = mpwx + weight[j]*(mZ[][:j]*invertsym(mZ[][:j]'mZ[][:j])*mZ[][:j]'vx - (betaj.GetLambda())*vx);
	delete betaj;
	}
	
	decl ve = vy-vx*betaFuller;

	decl vhat; vhat = mpwx.*ve;
	decl mvhat = invertsym(mpwx'vx)*(vhat'vhat) *invertsym(vx'mpwx);
	return sqrt(diagonal(mvhat))';
}

////

